import os
import sys
import subprocess as sbp
import numpy as np
import msm.utilities as utl


def run(code_name, input_name, *argv):
    """ Run codes

    :param code_name: Name of the code
    :type code_name: string

    :param input_name: Name of the input file
    :type input_name: string
    """
    # print 'prevabs' in code_name
    cmd = [code_name, ]
    if 'prevabs' in code_name:
        cmd.append('-i')
    cmd.append(input_name)
    cmd = cmd + list(argv)
    cmd = ' '.join(cmd)
    # print input_name
    try:
        # sys.stdout.write('> running ' + code_name + '...')
        # sys.stdout.flush()
        FNULL = open(os.devnull, 'w')
        sbp.call(cmd, stdout=FNULL, stderr=sbp.STDOUT)
    except:
        # sys.stdout.write('failed\n')
        # sys.stdout.flush()
        return
    # else:
    #     sys.stdout.write('done\n')
    #     sys.stdout.flush()


def calcCompliance(engconst, symm, reduced=False):
    s = np.zeros((6, 6))
    if symm == 'orthotropic':
        s[0, 0] = 1. / engconst['e1']
        s[1, 1] = 1. / engconst['e2']
        s[2, 2] = 1. / engconst['e3']
        s[0, 1] = s[1, 0] = -engconst['nu12'] / engconst['e1']
        s[0, 2] = s[2, 0] = -engconst['nu13'] / engconst['e2']
        s[1, 2] = s[2, 1] = -engconst['nu23'] / engconst['e3']
        s[3, 3] = 1. / engconst['g23']
        s[4, 4] = 1. / engconst['g13']
        s[5, 5] = 1. / engconst['g12']

    if reduced:
        s = s[[0, 1, 5], :][:, [0, 1, 5]]

    return s


def calcDirectionCosineMatrix(theta3):
    beta = np.zeros((3, 3))
    beta[0, 0] = beta[1, 1] = np.cos(np.deg2rad(theta3))
    beta[0, 1] = -np.sin(np.deg2rad(theta3))
    beta[1, 0] = np.sin(np.deg2rad(theta3))
    beta[2, 2] = 1.0
    return beta


def calcTransformationMatrix(theta3, reduced=False, derivative=False):
    beta = calcDirectionCosineMatrix(theta3)
    tm_e = np.zeros((6, 6))
    tm_s = np.zeros((6, 6))
    sub11 = beta**2
    sub12 = np.array([
        [beta[0, 1] * beta[0, 2], beta[0, 0] *
            beta[0, 2], beta[0, 0] * beta[0, 1]],
        [beta[1, 1] * beta[1, 2], beta[1, 0] *
            beta[1, 2], beta[1, 0] * beta[1, 1]],
        [beta[2, 1] * beta[2, 2], beta[2, 0] * beta[2, 2], beta[2, 0] * beta[2, 1]]
    ])
    sub21 = np.array([
        [beta[1, 0] * beta[2, 0], beta[1, 1] *
            beta[2, 1], beta[1, 2] * beta[2, 2]],
        [beta[0, 0] * beta[2, 0], beta[0, 1] *
            beta[2, 1], beta[0, 2] * beta[2, 2]],
        [beta[0, 0] * beta[1, 0], beta[0, 1] * beta[1, 1], beta[0, 2] * beta[1, 2]]
    ])
    sub22 = np.array([
        [beta[1, 2] * beta[2, 1] + beta[1, 1] * beta[2, 2], beta[1, 2] * beta[2, 0] +
            beta[1, 0] * beta[2, 2], beta[1, 1] * beta[2, 0] + beta[1, 0] * beta[2, 1]],
        [beta[0, 2] * beta[2, 1] + beta[0, 1] * beta[2, 2], beta[0, 2] * beta[2, 0] +
            beta[0, 0] * beta[2, 2], beta[0, 1] * beta[2, 0] + beta[0, 0] * beta[2, 1]],
        [beta[0, 2] * beta[1, 1] + beta[0, 1] * beta[1, 2], beta[0, 2] * beta[1, 0] +
            beta[0, 0] * beta[1, 2], beta[0, 1] * beta[1, 0] + beta[0, 0] * beta[1, 1]]
    ])

    tm_e[:3, :3] = sub11
    tm_e[:3, 3:] = sub12
    tm_e[3:, :3] = sub21 * 2.
    tm_e[3:, 3:] = sub22

    tm_s[:3, :3] = sub11
    tm_s[:3, 3:] = sub12 * 2.
    tm_s[3:, :3] = sub21
    tm_s[3:, 3:] = sub22

    if reduced:
        tm_e = tm_e[[0, 1, 5], :][:, [0, 1, 5]]
        tm_s = tm_s[[0, 1, 5], :][:, [0, 1, 5]]
        if derivative:
            c = np.cos(np.deg2rad(theta3))
            s = np.sin(np.deg2rad(theta3))
            d_tm_s = np.array([
                [-2. * c * s, 2. * c * s, -2. * (c**2 - s**2)],
                [2. * c * s, -2. * c * s, 2. * (c**2 - s**2)],
                [c**2 - s**2, s**2 - c**2, -4. * c * s]
            ])

    if derivative:
        return tm_s, d_tm_s
    else:
        return tm_s


def calcStiffnessPlaneStressReducedTransformed(qr, fo, derivative=False):
    if derivative:
        tm_s, d_tm_s = calcTransformationMatrix(
            fo, reduced=True, derivative=True)
    else:
        tm_s = calcTransformationMatrix(fo, reduced=True)
    qrt = np.dot(tm_s, np.dot(qr, tm_s.T))
    if derivative:
        d_qrt = np.dot(d_tm_s, np.dot(qr, tm_s.T)) + \
            np.dot(tm_s, np.dot(qr, d_tm_s.T))

    if derivative:
        return qrt, d_qrt
    else:
        return qrt


def calcABD(engconst, layupcode, plythick, derivative=False):
    reduced = True
    cmpl_r = calcCompliance(engconst, 'orthotropic', reduced)
    stff_r = np.linalg.inv(cmpl_r)
    layup = utl.parseLayupCode(layupcode)
    nply = len(layup)
    zbar = np.linspace(0., nply * plythick, nply, endpoint=False) - \
        nply * plythick / 2. + plythick / 2.
    a = b = d = np.zeros((3, 3))
    if derivative:
        da = db = dd = np.zeros((3, 3))
    for i, fo in enumerate(layup):
        if derivative:
            qrt, dqrt = calcStiffnessPlaneStressReducedTransformed(
                stff_r, fo, True)
        else:
            qrt = calcStiffnessPlaneStressReducedTransformed(stff_r, fo)

        a = a + qrt * plythick
        b = b + qrt * plythick * zbar[i]
        d = d + qrt * (plythick * zbar[i]**2 + plythick**3 / 12.)
        if derivative:
            da = da + dqrt * plythick
            db = db + dqrt * plythick * zbar[i]
            dd = dd + dqrt * (plythick * zbar[i]**2 + plythick**3 / 12.)

    if derivative:
        return a, b, d, da, db, dd
    else:
        return a, b, d
